@@ -1,8 +1,8 @@
 use crate::{
     self as ast, Alias, Arguments, BoolOp, BytesLiteral, CmpOp, Comprehension, Decorator,
-    ElifElseClause, ExceptHandler, Expr, ExprContext, FString, FStringElement, Keyword, MatchCase,
-    Operator, Parameter, Parameters, Pattern, PatternArguments, PatternKeyword, Stmt,
-    StringLiteral, TypeParam, TypeParamParamSpec, TypeParamTypeVar, TypeParamTypeVarTuple,
+    ElifElseClause, ExceptHandler, Expr, ExprContext, FString, InterpolatedStringElement, Keyword,
+    MatchCase, Operator, Parameter, Parameters, Pattern, PatternArguments, PatternKeyword, Stmt,
+    StringLiteral, TString, TypeParam, TypeParamParamSpec, TypeParamTypeVar, TypeParamTypeVarTuple,
     TypeParams, UnaryOp, WithItem,
 };
 
@@ -86,8 +86,14 @@ pub trait Transformer {
     fn visit_f_string(&self, f_string: &mut FString) {
         walk_f_string(self, f_string);
     }
-    fn visit_f_string_element(&self, f_string_element: &mut FStringElement) {
-        walk_f_string_element(self, f_string_element);
+    fn visit_interpolated_string_element(
+        &self,
+        interpolated_string_element: &mut InterpolatedStringElement,
+    ) {
+        walk_interpolated_string_element(self, interpolated_string_element);
+    }
+    fn visit_t_string(&self, t_string: &mut TString) {
+        walk_t_string(self, t_string);
     }
     fn visit_string_literal(&self, string_literal: &mut StringLiteral) {
         walk_string_literal(self, string_literal);
@@ -130,7 +136,7 @@ pub fn walk_stmt<V: Transformer + ?Sized>(visitor: &V, stmt: &mut Stmt) {
                 visitor.visit_type_params(type_params);
             }
             visitor.visit_parameters(parameters);
-            for expr in returns {
+            if let Some(expr) = returns {
                 visitor.visit_annotation(expr);
             }
             visitor.visit_body(body);
@@ -153,18 +159,27 @@ pub fn walk_stmt<V: Transformer + ?Sized>(visitor: &V, stmt: &mut Stmt) {
             }
             visitor.visit_body(body);
         }
-        Stmt::Return(ast::StmtReturn { value, range: _ }) => {
+        Stmt::Return(ast::StmtReturn {
+            value,
+            range: _,
+            node_index: _,
+        }) => {
             if let Some(expr) = value {
                 visitor.visit_expr(expr);
             }
         }
-        Stmt::Delete(ast::StmtDelete { targets, range: _ }) => {
+        Stmt::Delete(ast::StmtDelete {
+            targets,
+            range: _,
+            node_index: _,
+        }) => {
             for expr in targets {
                 visitor.visit_expr(expr);
             }
         }
         Stmt::TypeAlias(ast::StmtTypeAlias {
             range: _,
+            node_index: _,
             name,
             type_params,
             value,
@@ -186,6 +201,7 @@ pub fn walk_stmt<V: Transformer + ?Sized>(visitor: &V, stmt: &mut Stmt) {
             op,
             value,
             range: _,
+            node_index: _,
         }) => {
             visitor.visit_expr(value);
             visitor.visit_operator(op);
@@ -220,6 +236,7 @@ pub fn walk_stmt<V: Transformer + ?Sized>(visitor: &V, stmt: &mut Stmt) {
             body,
             orelse,
             range: _,
+            node_index: _,
         }) => {
             visitor.visit_expr(test);
             visitor.visit_body(body);
@@ -230,13 +247,11 @@ pub fn walk_stmt<V: Transformer + ?Sized>(visitor: &V, stmt: &mut Stmt) {
             body,
             elif_else_clauses,
             range: _,
+            node_index: _,
         }) => {
             visitor.visit_expr(test);
             visitor.visit_body(body);
             for clause in elif_else_clauses {
-                if let Some(test) = &mut clause.test {
-                    visitor.visit_expr(test);
-                }
                 walk_elif_else_clause(visitor, clause);
             }
         }
@@ -250,6 +265,7 @@ pub fn walk_stmt<V: Transformer + ?Sized>(visitor: &V, stmt: &mut Stmt) {
             subject,
             cases,
             range: _,
+            node_index: _,
         }) => {
             visitor.visit_expr(subject);
             for match_case in cases {
@@ -260,13 +276,14 @@ pub fn walk_stmt<V: Transformer + ?Sized>(visitor: &V, stmt: &mut Stmt) {
             exc,
             cause,
             range: _,
+            node_index: _,
         }) => {
             if let Some(expr) = exc {
                 visitor.visit_expr(expr);
-            };
+            }
             if let Some(expr) = cause {
                 visitor.visit_expr(expr);
-            };
+            }
         }
         Stmt::Try(ast::StmtTry {
             body,
@@ -275,6 +292,7 @@ pub fn walk_stmt<V: Transformer + ?Sized>(visitor: &V, stmt: &mut Stmt) {
             finalbody,
             is_star: _,
             range: _,
+            node_index: _,
         }) => {
             visitor.visit_body(body);
             for except_handler in handlers {
@@ -287,13 +305,18 @@ pub fn walk_stmt<V: Transformer + ?Sized>(visitor: &V, stmt: &mut Stmt) {
             test,
             msg,
             range: _,
+            node_index: _,
         }) => {
             visitor.visit_expr(test);
             if let Some(expr) = msg {
                 visitor.visit_expr(expr);
             }
         }
-        Stmt::Import(ast::StmtImport { names, range: _ }) => {
+        Stmt::Import(ast::StmtImport {
+            names,
+            range: _,
+            node_index: _,
+        }) => {
             for alias in names {
                 visitor.visit_alias(alias);
             }
@@ -305,7 +328,11 @@ pub fn walk_stmt<V: Transformer + ?Sized>(visitor: &V, stmt: &mut Stmt) {
         }
         Stmt::Global(_) => {}
         Stmt::Nonlocal(_) => {}
-        Stmt::Expr(ast::StmtExpr { value, range: _ }) => visitor.visit_expr(value),
+        Stmt::Expr(ast::StmtExpr {
+            value,
+            range: _,
+            node_index: _,
+        }) => visitor.visit_expr(value),
         Stmt::Pass(_) | Stmt::Break(_) | Stmt::Continue(_) | Stmt::IpyEscapeCommand(_) => {}
     }
 }
@@ -324,6 +351,7 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
             op,
             values,
             range: _,
+            node_index: _,
         }) => {
             visitor.visit_bool_op(op);
             for expr in values {
@@ -334,6 +362,7 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
             target,
             value,
             range: _,
+            node_index: _,
         }) => {
             visitor.visit_expr(value);
             visitor.visit_expr(target);
@@ -343,6 +372,7 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
             op,
             right,
             range: _,
+            node_index: _,
         }) => {
             visitor.visit_expr(left);
             visitor.visit_operator(op);
@@ -352,6 +382,7 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
             op,
             operand,
             range: _,
+            node_index: _,
         }) => {
             visitor.visit_unary_op(op);
             visitor.visit_expr(operand);
@@ -360,6 +391,7 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
             parameters,
             body,
             range: _,
+            node_index: _,
         }) => {
             if let Some(parameters) = parameters {
                 visitor.visit_parameters(parameters);
@@ -371,24 +403,29 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
             body,
             orelse,
             range: _,
+            node_index: _,
         }) => {
             visitor.visit_expr(test);
             visitor.visit_expr(body);
             visitor.visit_expr(orelse);
         }
         Expr::Dict(ast::ExprDict {
-            keys,
-            values,
+            items,
             range: _,
+            node_index: _,
         }) => {
-            for expr in keys.iter_mut().flatten() {
-                visitor.visit_expr(expr);
-            }
-            for expr in values {
-                visitor.visit_expr(expr);
+            for ast::DictItem { key, value } in items {
+                if let Some(key) = key {
+                    visitor.visit_expr(key);
+                }
+                visitor.visit_expr(value);
             }
         }
-        Expr::Set(ast::ExprSet { elts, range: _ }) => {
+        Expr::Set(ast::ExprSet {
+            elts,
+            range: _,
+            node_index: _,
+        }) => {
             for expr in elts {
                 visitor.visit_expr(expr);
             }
@@ -397,6 +434,7 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
             elt,
             generators,
             range: _,
+            node_index: _,
         }) => {
             for comprehension in generators {
                 visitor.visit_comprehension(comprehension);
@@ -407,6 +445,7 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
             elt,
             generators,
             range: _,
+            node_index: _,
         }) => {
             for comprehension in generators {
                 visitor.visit_comprehension(comprehension);
@@ -418,6 +457,7 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
             value,
             generators,
             range: _,
+            node_index: _,
         }) => {
             for comprehension in generators {
                 visitor.visit_comprehension(comprehension);
@@ -429,6 +469,7 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
             elt,
             generators,
             range: _,
+            node_index: _,
             parenthesized: _,
         }) => {
             for comprehension in generators {
@@ -436,18 +477,31 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
             }
             visitor.visit_expr(elt);
         }
-        Expr::Await(ast::ExprAwait { value, range: _ }) => visitor.visit_expr(value),
-        Expr::Yield(ast::ExprYield { value, range: _ }) => {
+        Expr::Await(ast::ExprAwait {
+            value,
+            range: _,
+            node_index: _,
+        }) => visitor.visit_expr(value),
+        Expr::Yield(ast::ExprYield {
+            value,
+            range: _,
+            node_index: _,
+        }) => {
             if let Some(expr) = value {
                 visitor.visit_expr(expr);
             }
         }
-        Expr::YieldFrom(ast::ExprYieldFrom { value, range: _ }) => visitor.visit_expr(value),
+        Expr::YieldFrom(ast::ExprYieldFrom {
+            value,
+            range: _,
+            node_index: _,
+        }) => visitor.visit_expr(value),
         Expr::Compare(ast::ExprCompare {
             left,
             ops,
             comparators,
             range: _,
+            node_index: _,
         }) => {
             visitor.visit_expr(left);
             for cmp_op in &mut **ops {
@@ -461,6 +515,7 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
             func,
             arguments,
             range: _,
+            node_index: _,
         }) => {
             visitor.visit_expr(func);
             visitor.visit_arguments(arguments);
@@ -477,6 +532,11 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
                 }
             }
         }
+        Expr::TString(ast::ExprTString { value, .. }) => {
+            for t_string in value.iter_mut() {
+                visitor.visit_t_string(t_string);
+            }
+        }
         Expr::StringLiteral(ast::ExprStringLiteral { value, .. }) => {
             for string_literal in value.iter_mut() {
                 visitor.visit_string_literal(string_literal);
@@ -500,6 +560,7 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
             slice,
             ctx,
             range: _,
+            node_index: _,
         }) => {
             visitor.visit_expr(value);
             visitor.visit_expr(slice);
@@ -509,6 +570,7 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
             value,
             ctx,
             range: _,
+            node_index: _,
         }) => {
             visitor.visit_expr(value);
             visitor.visit_expr_context(ctx);
@@ -520,6 +582,7 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
             elts,
             ctx,
             range: _,
+            node_index: _,
         }) => {
             for expr in elts {
                 visitor.visit_expr(expr);
@@ -530,6 +593,7 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
             elts,
             ctx,
             range: _,
+            node_index: _,
             parenthesized: _,
         }) => {
             for expr in elts {
@@ -542,6 +606,7 @@ pub fn walk_expr<V: Transformer + ?Sized>(visitor: &V, expr: &mut Expr) {
             upper,
             step,
             range: _,
+            node_index: _,
         }) => {
             if let Some(expr) = lower {
                 visitor.visit_expr(expr);
@@ -580,13 +645,13 @@ pub fn walk_except_handler<V: Transformer + ?Sized>(
 }
 
 pub fn walk_arguments<V: Transformer + ?Sized>(visitor: &V, arguments: &mut Arguments) {
-    // Note that the there might be keywords before the last arg, e.g. in
+    // Note that there might be keywords before the last arg, e.g. in
     // f(*args, a=2, *args2, **kwargs)`, but we follow Python in evaluating first `args` and then
     // `keywords`. See also [Arguments::arguments_source_order`].
-    for arg in arguments.args.iter_mut() {
+    for arg in &mut *arguments.args {
         visitor.visit_expr(arg);
     }
-    for keyword in arguments.keywords.iter_mut() {
+    for keyword in &mut *arguments.keywords {
         visitor.visit_keyword(keyword);
     }
 }
@@ -656,6 +721,7 @@ pub fn walk_type_param<V: Transformer + ?Sized>(visitor: &V, type_param: &mut Ty
             default,
             name: _,
             range: _,
+            node_index: _,
         }) => {
             if let Some(expr) = bound {
                 visitor.visit_expr(expr);
@@ -668,6 +734,7 @@ pub fn walk_type_param<V: Transformer + ?Sized>(visitor: &V, type_param: &mut Ty
             default,
             name: _,
             range: _,
+            node_index: _,
         }) => {
             if let Some(expr) = default {
                 visitor.visit_expr(expr);
@@ -677,6 +744,7 @@ pub fn walk_type_param<V: Transformer + ?Sized>(visitor: &V, type_param: &mut Ty
             default,
             name: _,
             range: _,
+            node_index: _,
         }) => {
             if let Some(expr) = default {
                 visitor.visit_expr(expr);
@@ -751,29 +819,35 @@ pub fn walk_pattern_keyword<V: Transformer + ?Sized>(
 
 pub fn walk_f_string<V: Transformer + ?Sized>(visitor: &V, f_string: &mut FString) {
     for element in &mut f_string.elements {
-        visitor.visit_f_string_element(element);
+        visitor.visit_interpolated_string_element(element);
     }
 }
 
-pub fn walk_f_string_element<V: Transformer + ?Sized>(
+pub fn walk_interpolated_string_element<V: Transformer + ?Sized>(
     visitor: &V,
-    f_string_element: &mut FStringElement,
+    interpolated_string_element: &mut InterpolatedStringElement,
 ) {
-    if let ast::FStringElement::Expression(ast::FStringExpressionElement {
+    if let ast::InterpolatedStringElement::Interpolation(ast::InterpolatedElement {
         expression,
         format_spec,
         ..
-    }) = f_string_element
+    }) = interpolated_string_element
     {
         visitor.visit_expr(expression);
         if let Some(format_spec) = format_spec {
             for spec_element in &mut format_spec.elements {
-                visitor.visit_f_string_element(spec_element);
+                visitor.visit_interpolated_string_element(spec_element);
             }
         }
     }
 }
 
+pub fn walk_t_string<V: Transformer + ?Sized>(visitor: &V, t_string: &mut TString) {
+    for element in &mut t_string.elements {
+        visitor.visit_interpolated_string_element(element);
+    }
+}
+
 pub fn walk_expr_context<V: Transformer + ?Sized>(_visitor: &V, _expr_context: &mut ExprContext) {}
 
 pub fn walk_bool_op<V: Transformer + ?Sized>(_visitor: &V, _bool_op: &mut BoolOp) {}
