@@ -244,7 +244,7 @@ impl ruff_db::Db for CorpusDb {
 
 #[salsa::db]
 impl ty_python_semantic::Db for CorpusDb {
-    fn is_file_open(&self, file: File) -> bool {
+    fn should_check_file(&self, file: File) -> bool {
         !file.path(self).is_vendored_path()
     }
 
@@ -255,6 +255,10 @@ impl ty_python_semantic::Db for CorpusDb {
     fn lint_registry(&self) -> &LintRegistry {
         default_lint_registry()
     }
+
+    fn verbose(&self) -> bool {
+        false
+    }
 }
 
 #[salsa::db]
